//-----------------------------------------------------------------------------
#ifndef drawingCanvasH
#define drawingCanvasH drawingCanvasH
//-----------------------------------------------------------------------------
#include <wx/scrolwin.h>
#include <mvIMPACT_CPP/mvIMPACT_acquire.h>

//-----------------------------------------------------------------------------
template<typename T>
inline const T& saveAssign( const T& val, const T& min, const T& max )
//-----------------------------------------------------------------------------
{
	return ( ( val > max ) ? max : ( val < min ) ? min : val );
}

//-----------------------------------------------------------------------------
struct AOI
//-----------------------------------------------------------------------------
{
	wxRect		m_rect;
	wxColour	m_colour;
	wxString	m_description;
	AOI( const wxRect& r, const wxColour& c, const wxString& desc ) : m_rect(r), m_colour(c), m_description(desc) {}
};

//-----------------------------------------------------------------------------
class DrawingCanvas : public wxScrolledWindow
//-----------------------------------------------------------------------------
{
	DECLARE_DYNAMIC_CLASS(DrawingCanvas)
	bool				m_boActive;
protected:
	mutable wxCriticalSection	m_critSect;
public:
	DrawingCanvas() {}
	DrawingCanvas( wxWindow* parent, wxWindowID id = -1, const wxPoint& pos = wxDefaultPosition,
				   const wxSize& size = wxDefaultSize, long style = wxSUNKEN_BORDER,
				   const wxString& name = wxT("DrawingCanvas"), bool boActive = false );
	bool				IsActive( void ) const { return m_boActive; }
	virtual void		SetActive( bool boActive );
};

class ImageCanvas;

//-----------------------------------------------------------------------------
struct PlotPoint
//-----------------------------------------------------------------------------
{
	int cnt_;
	int val_;
	PlotPoint( int cnt, int val ) : cnt_(cnt), val_(val) {}
	PlotPoint() : cnt_(0), val_(0) {}
};

//-----------------------------------------------------------------------------
struct NamedColour
//-----------------------------------------------------------------------------
{
	const wxColour* pColour_;
	wxString description_;
	NamedColour() : pColour_(0), description_(wxT("")) {}
	NamedColour( const wxColour* p, const wxString& d ) : pColour_(p), description_(d) {}
};

//-----------------------------------------------------------------------------
class PlotCanvas : public DrawingCanvas
//-----------------------------------------------------------------------------
{
	int				m_BorderWidth;
protected:
	int				m_ImageCount;
	int				m_UpdateFrequency;
public:
	PlotCanvas() {}
	PlotCanvas( wxWindow* parent, wxWindowID id = -1, const wxPoint& pos = wxDefaultPosition,
				const wxSize& size = wxDefaultSize, long style = wxSUNKEN_BORDER,
				const wxString& name = wxT("PlotCanvas"), bool boActive = false );
	bool			SetUpdateFrequency( int frequency );
	int				GetBorderWidth( void ) const { return m_BorderWidth; }
	void			SetBorderWidth( int borderWidth );
	//-----------------------------------------------------------------------------
	enum TUpdateLimit
	//-----------------------------------------------------------------------------
	{
		ulMin = 1,
		ulMax = 100
	};
};

//-----------------------------------------------------------------------------
class ImageAnalysisPlotCanvas : public PlotCanvas
//-----------------------------------------------------------------------------
{
	wxColour		m_AOIColour;
	ImageCanvas*	m_pImageCanvas;
	wxString		m_ConfigName;
	bool			m_boAOIFullMode;
	bool			SetAOI( const mvIMPACT::acquire::ImageBuffer* pIB, int x, int y, int w, int h );
protected:
	//-----------------------------------------------------------------------------
	enum
	//-----------------------------------------------------------------------------
	{
		VAL_COUNT_8_BIT = 256,
		VAL_COUNT_10_BIT = VAL_COUNT_8_BIT << 2,
		VAL_COUNT_12_BIT = VAL_COUNT_8_BIT << 4,
		VAL_COUNT_14_BIT = VAL_COUNT_8_BIT << 6,
		VAL_COUNT_16_BIT = VAL_COUNT_8_BIT << 8
	};
	//-----------------------------------------------------------------------------
	enum TPlane
	//-----------------------------------------------------------------------------
	{
		pRed = 0,
		pY = 0,
		pChannel0Hor = 0,
		pGreen = 1,
		pU = 1,
		pChannel0Ver = 1,
		pBlue = 2,
		pV = 2,
		pChannel1Hor = 2,
		pChannel1Ver = 3,
		pChannel2Hor = 4,
		pChannel2Ver = 5,
		PLANE_CNT
	};
	unsigned int	m_valCount;
	int				m_AOIx;
	int				m_AOIy;
	int				m_AOIw;
	int				m_AOIh;
	AOI*			m_pAOI;
	int				m_ChannelCount;
	mvIMPACT::acquire::TImageBufferPixelFormat m_pixelFormat;
	NamedColour		m_Pens[PLANE_CNT];
	bool			MustUpdate( const mvIMPACT::acquire::ImageBuffer* pIB, int plotChannelCount, int x, int y, int w, int h, bool boForceRefresh, bool* pboAOIChanged = 0, bool* pboImageFormatChanged = 0 );
public:
	ImageAnalysisPlotCanvas() {}
	ImageAnalysisPlotCanvas( wxWindow* parent, const wxString& configName, wxWindowID id = -1, const wxPoint& pos = wxDefaultPosition,
				const wxSize& size = wxDefaultSize, long style = wxSUNKEN_BORDER,
				const wxString& name = wxT("ImageAnalysisPlotCanvas"), bool boActive = false, int channelCount = 0 );
	const wxString&	GetConfigName( void ) const { return m_ConfigName; }
	virtual void	RefreshData( const mvIMPACT::acquire::ImageBuffer* pIB, int x = -1, int y = -1, int w = -1, int h = -1, bool boForceRefresh = false ) = 0;
	virtual void	SetActive( bool boActive );
	bool			IsAOIFullModeActive( void ) const { return m_boAOIFullMode; }
	void			SetAOIFullMode( bool boActive );
	void			SetAOIColour( const wxColour& );
	virtual bool	HasPercentageWindow( void ) const { return false; }
	void			RegisterImageCanvas( ImageCanvas* p );
};

#endif // drawingCanvasH
