//-----------------------------------------------------------------------------
#ifndef histogramCanvasH
#define histogramCanvasH histogramCanvasH
//-----------------------------------------------------------------------------
#include <wx/dcclient.h>
#include "DrawingCanvas.h"

//-----------------------------------------------------------------------------
class HistogramCanvas : public ImageAnalysisPlotCanvas
//-----------------------------------------------------------------------------
{
	int				m_DrawStart_percent;
	int				m_DrawWindow_percent;
protected:
	int**			m_ppHistogramBuffer;
	double*			m_pHistogramAverage;
	PlotPoint		m_CurrentMax;
	void			CalculateMaxValue( void );
	void			DeallocateHistogramBuffer( void );
	void			PrepareHistogramBuffer( bool boImageFormatChanged, int channelCount, mvIMPACT::acquire::TImageBufferPixelFormat pixelFormat );
	void			DrawHistogramLine( wxPaintDC& dc, int h, double scaleX, double scaleY, TPlane plane );
	DECLARE_EVENT_TABLE()
public:
	HistogramCanvas() {}
	HistogramCanvas( wxWindow* parent, const wxString& configName, wxWindowID id = -1, const wxPoint& pos = wxDefaultPosition,
				     const wxSize& size = wxDefaultSize, long style = wxSUNKEN_BORDER,
				     const wxString& name = wxT("Histogram"), bool boActive = false );
	~HistogramCanvas();
	void			OnPaint( wxPaintEvent & );
	virtual bool	HasPercentageWindow( void ) const { return true; }
	void			SetDrawStartPercent( int value_percent );
	void			SetDrawWindowWidthPercent( int value_percent );
};

//-----------------------------------------------------------------------------
class PixelHistogramCanvas : public HistogramCanvas
//-----------------------------------------------------------------------------
{
	DECLARE_DYNAMIC_CLASS(PixelHistogramCanvas)
public:
	PixelHistogramCanvas() {}
	PixelHistogramCanvas( wxWindow* parent, wxWindowID id = -1, const wxPoint& pos = wxDefaultPosition,
				     const wxSize& size = wxDefaultSize, long style = wxSUNKEN_BORDER,
				     const wxString& name = wxT("Pixel Histogram"), bool boActive = false );
	virtual void			RefreshData( const mvIMPACT::acquire::ImageBuffer* pIB, int x = -1, int y = -1, int w = -1, int h = -1, bool boForceRefresh = false );
};

//-----------------------------------------------------------------------------
class NoiseHistogramCanvas : public HistogramCanvas
//-----------------------------------------------------------------------------
{
	DECLARE_DYNAMIC_CLASS(NoiseHistogramCanvas)
public:
	NoiseHistogramCanvas() {}
	NoiseHistogramCanvas( wxWindow* parent, wxWindowID id = -1, const wxPoint& pos = wxDefaultPosition,
				     const wxSize& size = wxDefaultSize, long style = wxSUNKEN_BORDER,
				     const wxString& name = wxT("Noise Histogram"), bool boActive = false );
	virtual void	RefreshData( const mvIMPACT::acquire::ImageBuffer* pIB, int x = -1, int y = -1, int w = -1, int h = -1, bool boForceRefresh = false );
};

#endif // histogramCanvasH
