//-----------------------------------------------------------------------------
#ifndef DeviceConfigureFrameH
#define DeviceConfigureFrameH DeviceConfigureFrameH
//-----------------------------------------------------------------------------
#include "DevCtrl.h"
#include <mvIMPACT_CPP/mvIMPACT_acquire.h>
#ifdef BUILD_WITH_PLUGIN_SUPPORT
#	include <mvPluginManager/Include/mvPluginManager.h>
#endif // #ifdef BUILD_WITH_PLUGIN_SUPPORT
#include <wx/wx.h>

class DevicePropertyHandler;
class ImageAnalysisPlotCanvas;
class ImageCanvas;
class InfoPlotCanvas;

class wxConfigBase;
class wxNotebook;
class wxPropertyGrid;
class wxPropertyGridEvent;
class wxSplitterEvent;
class wxSplitterWindow;
class wxSpinCtrl;

DECLARE_EVENT_TYPE(imageReadyEvent, -1)
DECLARE_EVENT_TYPE(imageSkippedEvent, -2)
DECLARE_EVENT_TYPE(toggleDisplayArea, -3)
DECLARE_EVENT_TYPE(liveModeAborted, -4)
DECLARE_EVENT_TYPE(eventNotified, -5)
DECLARE_EVENT_TYPE(sequenceReadyEvent, -6)
DECLARE_EVENT_TYPE(imageInfoEvent, -7)

//-----------------------------------------------------------------------------
struct ImageAnalysisPlotControls
//-----------------------------------------------------------------------------
{
	wxCheckBox*					m_pCBEnablePlot;
	wxCheckBox*					m_pCBAOIFullMode;
	wxSpinCtrl*					m_pSCAOIx;
	wxSpinCtrl*					m_pSCAOIy;
	wxSpinCtrl*					m_pSCAOIw;
	wxSpinCtrl*					m_pSCAOIh;
	wxSpinCtrl*					m_pSCDrawStart_percent;
	wxSpinCtrl*					m_pSCDrawWindow_percent;
	wxSpinCtrl*					m_pSCUpdateSpeed;
	ImageAnalysisPlotCanvas*	m_pPlotCanvas;
	void						Load( wxConfigBase* pConfig );
	void						Save( wxConfigBase* pConfig );
	void						UpdateControls( void );
	explicit ImageAnalysisPlotControls() : m_pCBEnablePlot(0), m_pCBAOIFullMode(0),
		m_pSCAOIx(0), m_pSCAOIy(0), m_pSCAOIw(0), m_pSCAOIh(0), m_pSCDrawStart_percent(0),
		m_pSCDrawWindow_percent(0), m_pSCUpdateSpeed(0), m_pPlotCanvas(0) {}
};

//-----------------------------------------------------------------------------
class PropViewFrame : public wxFrame
//-----------------------------------------------------------------------------
{
public:
	PropViewFrame(const wxString& title, const wxPoint& pos, const wxSize& size, const wxString& cmdLine );
	~PropViewFrame();
	void PostInit();
protected:
	// event handlers (these functions should _not_ be virtual)
	void OnAbout( wxCommandEvent& e );
	void OnActivateDisplay( wxCommandEvent& e );
	void OnBtnAbort( wxCommandEvent& e );
	void OnBtnLive( wxCommandEvent& e );
	void OnBtnRecord( wxCommandEvent& e );
	void OnBtnForward( wxCommandEvent& e );
	void OnBtnBackward( wxCommandEvent& e );
	void OnUseDevice( wxCommandEvent& e );
	void OnBtnSnap( wxCommandEvent& e );
	void OnCBEnableAnalysisPlot( wxCommandEvent& e );
	void OnAnalysisPlotCBAOIFullMode( wxCommandEvent& e );
	void OnCBEnableInfoPlot( wxCommandEvent& );
	void OnClose( wxCloseEvent& e );
	void OnConnectedOnly( wxCommandEvent& e );
	void OnDevComboTextChanged( wxCommandEvent& e );
	void OnEventNotified( wxCommandEvent& e );
	void OnExportActiveDevice( wxCommandEvent& e );
	void OnFitToDisplay( wxCommandEvent& e );
	void OnAnalysisPlotAOIxTextChanged( wxCommandEvent& e );
	void OnAnalysisPlotAOIyTextChanged( wxCommandEvent& e );
	void OnAnalysisPlotAOIwTextChanged( wxCommandEvent& e );
	void OnAnalysisPlotAOIhTextChanged( wxCommandEvent& e );
	void OnAnalysisPlotDrawStartChanged( wxCommandEvent& e );
	void OnAnalysisPlotDrawWindowChanged( wxCommandEvent& e );
	void OnAnalysisPlotUpdateSpeedTextChanged( wxCommandEvent& e );
	void OnImageInfo( wxCommandEvent& e );
	void OnImageReady( wxCommandEvent& e );
	void OnImageSkipped( wxCommandEvent& e );
	void OnImportActiveDevice( wxCommandEvent& e );
	void OnInfoPlot_PlotSelectionComboTextChanged( wxCommandEvent& e );
	void OnInfoPlotUpdateHistoryDepth( wxCommandEvent& e );
	void OnInfoPlotUpdateSpeedTextChanged( wxCommandEvent& e );
	void OnLiveModeAborted( wxCommandEvent& e );
	void OnLoadActiveDevice( wxCommandEvent& e );
	void OnLoadFromDefault( wxCommandEvent& e );
	void OnPaint( wxPaintEvent& e );
	void OnPopUpMethExec( wxCommandEvent& e );
	void OnPopUpPropRestoreDefault( wxCommandEvent& e );
	void OnPopUpPropAppendValue( wxCommandEvent& e );
	void OnPopUpPropDeleteValue( wxCommandEvent& e );
	void OnPropertyChanged( wxPropertyGridEvent& e );
	void OnPropertyRightClicked( wxPropertyGridEvent& e );
	void OnQuit( wxCommandEvent& e );
	void OnSaveActiveDevice( wxCommandEvent& e );
	void OnSaveImage( wxCommandEvent& e );
	void OnSaveImageSequence( wxCommandEvent& e );
	void OnSaveToDefault( wxCommandEvent& e );
	void OnSaveSystemToDefault( wxCommandEvent& e );
	void OnSequenceReady( wxCommandEvent& e );
	void OnSettingsDisplayLogWindow( wxCommandEvent& );
	void OnSettingsPropGrid_Show( wxCommandEvent& );
	void OnSettingsPropGrid_StandardView( wxCommandEvent& );
	void OnSettingsPropGrid_DevelopersView( wxCommandEvent& );
	void OnShowDisplayMessagesAsOverlay( wxCommandEvent& e );
	void OnShowRequestInfosAsOverlay( wxCommandEvent& e );
	void OnSize( wxSizeEvent& e );
	void OnSplitterSashPosChanged( wxSplitterEvent& e );
	void OnTimer( wxTimerEvent& e );
	void OnToggleDisplayWindowSize( wxCommandEvent& e );
	void OnUpdateDeviceList( wxCommandEvent& e );
	void SetupDlgControls( void );
private:
	//-----------------------------------------------------------------------------
	enum TTimerEvent
	//-----------------------------------------------------------------------------
	{
		teListUpdate
	};
	//-----------------------------------------------------------------------------
	// IDs for the controls and the menu commands
	enum TMenuItem
	//-----------------------------------------------------------------------------
	{
		miQuit = 1,
		miHelp_About,
		miAction_UseDevice,
		miActionSave_ToDefault,
		miActionSave_SystemToDefault,
		miActionSave_ActiveDevice,
		miActionSave_ExportActiveDevice,
		miActionLoad_FromDefault,
		miActionLoad_ActiveDevice,
		miActionLoad_ImportActiveDevice,
		miAction_SaveImage,
		miAction_SaveImageSequence,
		miAction_UpdateDeviceList,
		miPopUpMethExec,
		miPopUpPropRestoreDefault,
		miPopUpPropAppendValue,
		miPopUpPropDeleteValue,
		miCapture_Live,
		miCapture_Snap,
		miCapture_Abort,
		miCapture_Record,
		miCapture_Forward,
		miCapture_Backward,
		miSettingsDisplay_FitToScreen,
		miSettingsDisplay_OverlayMessages,
		miSettingsDisplay_RequestInfoAsOverlay,
		miSettingsDisplay_Active,
		miSettings_ShowConnectedDevicesOnly,
		miSettings_ShowLogWindow,
		miSettingsPropGrid_Show,
		miSettingsPropGrid_StandardView,
		miSettingsPropGrid_DevelopersView
	};
	//-----------------------------------------------------------------------------
	enum TWidgetIDs
	//-----------------------------------------------------------------------------
	{
		widBtnAbort = 1,
		widBtnLive,
		widBtnUseDevice,
		widBtnSnap,
		widStatusBar,
		widVerSplitter,
		widDevCombo,
		widHorSplitter,
		widLogHistogramSplitter,
		widMainFrame,
		widPropGrid,
		widInfoPlot_PlotSelectionCombo,
		widCBEnableInfoPlot,
		widSCInfoPlotUpdateSpeed,
		widSCInfoPlotHistoryDepth,

		widPixelHistogram = 0x100,
		widNoiseHistogram = 0x200,
		widLineProfile = 0x300
	};
	//-----------------------------------------------------------------------------
	enum TImageAnalysisPlotIDs
	//-----------------------------------------------------------------------------
	{
		iapidCBEnable,
		iapidCBAOIFullMode,
		iapidSCAOIx,
		iapidSCAOIy,
		iapidSCAOIw,
		iapidSCAOIh,
		iapidSCDrawStart_percent,
		iapidSCDrawWindow_percent,
		iapidSCUpdateSpeed
	};
	//-----------------------------------------------------------------------------
	enum TImageAnalysisPlot
	//-----------------------------------------------------------------------------
	{
		iapPixelHistogram,
		iapNoiseHistogram,
		iapLineProfile,
		iapLAST
	};
	//-----------------------------------------------------------------------------
	enum
	//-----------------------------------------------------------------------------
	{
		TIMER_PERIOD = 750
	};
	//-----------------------------------------------------------------------------
	enum TStatusBarField
	//-----------------------------------------------------------------------------
	{
		sbfFPS,
		sbfFrameCount,
		sbfImageFormat,
		sbfErrorCount,
		sbfExposeTime,
		sbfGain,
		NO_OF_STATUSBAR_FIELDS
	};
	ImageAnalysisPlotControls			m_ImageAnalysisPlots[iapLAST];
	wxPanel* CreateImageAnalysisPlotControls( wxWindow* pParent, int windowIDOffset );

	bool								m_boDisplayWindowMaximized;
	mvIMPACT::acquire::ImageBufferDesc	m_CurrentImage;
	wxString							m_CurrentPixelFormat;
	wxString							m_CurrentRequestResult;
	RequestInfoData						m_CurrentRequestInfo;
	unsigned int						m_DeviceListChangedCounter;
	int									m_HorizontalSplitterPos;
	wxTimer								m_ListUpdateTimer;
	const wxString						m_NoDevStr;
	wxCheckBox*							m_pCBEnableInfoPlot;
	wxComboBox*							m_pDevCombo;
	DevicePropertyHandler*				m_pDevPropHandler;
	ImageCanvas*						m_pDisplayArea;
	wxSplitterWindow*					m_pHorizontalSplitter;
	InfoPlotCanvas*						m_pInfoPlotArea;
	wxComboBox*							m_pInfoPlotSelectionCombo;
	wxTextCtrl*							m_pLogWindow;
	wxNotebook*							m_pLowerRightWindow;
#ifdef BUILD_WITH_PLUGIN_SUPPORT
	PluginManager						m_pluginManager;
#endif // #ifdef BUILD_WITH_PLUGIN_SUPPORT
	wxMenuItem*							m_pMIAction_Use;
	wxMenuItem*							m_pMIAction_Save_ToDefault;
	wxMenuItem*							m_pMIAction_Save_SystemToDefault;
	wxMenuItem*							m_pMIAction_Save_ActiveDevice;
	wxMenuItem*							m_pMIAction_Save_ExportActiveDevice;
	wxMenuItem*							m_pMIAction_Load_FromDefault;
	wxMenuItem*							m_pMIAction_Load_ActiveDevice;
	wxMenuItem*							m_pMIAction_Load_ImportActiveDevice;
	wxMenuItem*							m_pMIAction_SaveImage;
	wxMenuItem*							m_pMIAction_SaveImageSequence;
	wxMenuItem*							m_pMIAction_UpdateDeviceList;
	wxMenuItem*							m_pMICapture_Live;
	wxMenuItem*							m_pMICapture_Snap;
	wxMenuItem*							m_pMICapture_Abort;
	wxMenuItem*							m_pMICapture_Record;
	wxMenuItem*							m_pMICapture_Forward;
	wxMenuItem*							m_pMICapture_Backward;
	wxMenuItem*							m_pMISettings_Display_FitToScreen;
	wxMenuItem*							m_pMISettings_Display_ShowMessagesAsOverlay;
	wxMenuItem*							m_pMISettings_Display_ShowRequestInfoAsOverlay;
	wxMenuItem*							m_pMISettings_Display_Active;
	wxMenuItem*							m_pMISettings_ShowConnectedOnly;
	wxMenuItem*							m_pMISettings_Display_LogWindow;
	wxMenuItem*							m_pMISettings_PropGrid_Show;
	wxMenuItem*							m_pMISettings_PropGrid_StandardView;
	wxMenuItem*							m_pMISettings_PropGrid_DevelopersView;
	wxToolBar*							m_pToolBar;
	wxPropertyGrid*						m_pPropGrid;
	wxSpinCtrl*							m_pSCInfoPlotHistoryDepth;
	wxSpinCtrl*							m_pSCInfoPlotUpdateSpeed;
	wxStatusBar*						m_pStatusBar;
	wxSplitterWindow*					m_pVerticalSplitter;
	static const wxString				m_UNKNOWN_STRING;
	int									m_VerticalSplitterPos;
	DevicePropertyHandler::TViewMode	m_ViewMode;
	void								Abort( void );
	void								ConfigureLive( void );
	void								Deinit( void );
	int									GetAnalysisControlIndex( int windowID ) const;
	void								SaveImage( const wxString& filenameAndPath, const wxString& extension );
	void								SetupDisplayLogSplitter( bool boSplit );
	void								SetupUpdateFrequencies( bool boCurrentValue );
	void								SetupVerSplitter( bool boSplit );
	void								UpdateDeviceComboBox( void );
	void								UpdateDeviceFromComboBox( void );
	void								UpdateData( bool boShowInfo = false );
	void								UpdateStatusBar( void );
	void								UpdateTitle( void );
	void								UpdatePropGridSplitter( void );
	void								ToggleCurrentDevice( void );
	void								WriteLogMessage( const wxString& msg );
	void								WriteLogMessage( const wxString& msg, const wxTextAttr& style);
	void								WriteErrorMessage( const wxString& msg );
	// any class wishing to process wxWidgets events must use this macro
	DECLARE_EVENT_TABLE()
};

#endif // DeviceConfigureFrameH
