#ifdef linux
#include "PoCLControl.h"
#include "hyperion.h"
#else
#include "stddcls.h"
#include "PoCLControl.h"
//#include "PoCLControl.tmh"
#endif

#define DBG 0
#ifdef linux
void jit_timer_pocl( unsigned long arg );

#define CANCEL_TIMER() del_timer( &ppo->timer )

#define ADD_TIMER(timer_expires_msec)\
{\
	unsigned long timeout_jiffies = msecs_to_jiffies( timer_expires_msec );\
	mod_timer( &ppo->timer, (jiffies + timeout_jiffies) );\
}

#if DBG
#define dbg_print(msg) printk msg
#else
#define dbg_print(msg)
#endif

#else

VOID DpcForPoCL( IN PKDPC Dpc, IN PVOID DeferredContext, IN PVOID SystemArgument1, IN PVOID SystemArgument2 );

#define CANCEL_TIMER() KeCancelTimer( &ppo->Timer )

#define ADD_TIMER(timer_expires_msec)\
{\
	LARGE_INTEGER lTimeout;\
	lTimeout.QuadPart = -10000;\
	lTimeout.QuadPart *= timer_expires_msec;\
	KeSetTimer( &ppo->Timer, lTimeout, &ppo->PoCLDpc );\
}

#if DBG
#define dbg_print(msg) KdPrint(msg)
#else
#define dbg_print(msg)
#endif

#endif

#if DBG
static char *PoCLActionstring[] = {
	"epcsConnectionSense",
	"epcsCameraCableDetect",
	"epcsClockDetect", 
	"res0",
	"res1",
};
#endif

#define ENABLE_INTERRUPT 0
//----------------------------------------------------------------------------------
int CreatePoCLObject( POCL_OBJECT *ppo, void *base, unsigned int reg_index )
//----------------------------------------------------------------------------------
{
	int result = FALSE;

	dbg_print(( " %s, %p, %p, id %d\n", __FUNCTION__, ppo, base, id ));
	if( ppo != NULL && base != NULL )
	{
		ppo->register_index = reg_index;
		ppo->Base = (POCL_CONTROL *)base;
		if( ppo->Base->version == -1 )
		{
			return FALSE;
		}
#ifdef linux
		init_timer( &ppo->timer );
		ppo->timer.function = jit_timer_pocl;
		ppo->timer.expires = 0;
		ppo->timer.data = (unsigned long)ppo;
#else
		KeInitializeDpc( &ppo->PoCLDpc, DpcForPoCL, (PVOID)ppo );
		KeInitializeTimer( &ppo->Timer );
#endif
		ppo->Action = epcsConnectionSense;
		ppo->DetectedCamera = eptNoCameraDetected;
		ppo->Base->control = CL_CLK_OBSERVE;
#if ENABLE_INTERRUPT
		ppo->Base->interrupt_enable = (OVERLOAD_INTR | CL_CLK_HALT_INTR);
#else
		ppo->Base->interrupt_enable = 0;
#endif
		ppo->InitDone = TRUE;
		ppo->CLClockLost = TRUE;
		HandlePoCLEvent( ppo );
	}
	return result;
}

//----------------------------------------------------------------------------------
void ClosePoCL( POCL_OBJECT *ppo )
//----------------------------------------------------------------------------------
{
	if( ppo->InitDone )
	{
		ppo->InitDone = FALSE;
		CANCEL_TIMER();
		ppo->Base->interrupt_enable = 0;
	}
}

//----------------------------------------------------------------------------------
void HandlePoCLEvent( POCL_OBJECT *ppo )
//----------------------------------------------------------------------------------
{
	unsigned int pocl_state;
	while( 1 )
	{
		//dbg_print(( " %s, id %d, %s\n", __FUNCTION__, ppo->Id, PoCLActionstring[ppo->Action] ));
		switch( ppo->Action )
		{
		case epcsConnectionSense:
			{
#ifdef linux
				ppo->Base->control = cpu_to_le32( CL_CLK_OBSERVE );
#else
				ppo->Base->control = CL_CLK_OBSERVE;
#endif
				ppo->Action = epcsCameraCableDetect;
				ADD_TIMER( 500 ); //this timeout is not specified but necessary to avoid to give the connection sense to much system resource
				break;
			}
		case epcsCameraCableDetect:
			{
				ppo->Action = epcsClockDetect;
#ifdef linux
				pocl_state = le32_to_cpu( ppo->Base->status );
#else
				pocl_state = ppo->Base->status;
#endif
				dbg_print(( " %s, id %d, status 0x%x\n", __FUNCTION__, ppo->Id, pocl_state ));
				ppo->DetectedCamera = pocl_state & STAT_THRESHOLD;
				switch( ppo->DetectedCamera )
				{
				case eptNonPoCLCamera:
#ifdef linux
					ppo->Base->control = cpu_to_le32( CL_CLK_OBSERVE | NON_POCL_CAMERA_ENABLE );
#else
					ppo->Base->control = (CL_CLK_OBSERVE | NON_POCL_CAMERA_ENABLE);
#endif
					ADD_TIMER( 3000 );
					break;
				case eptPoCLCamera:
#ifdef linux
					ppo->Base->control = cpu_to_le32( CL_CLK_OBSERVE | POCL_CAMERA_ENABLE );
#else
					ppo->Base->control = (CL_CLK_OBSERVE | POCL_CAMERA_ENABLE);
#endif
					ADD_TIMER( 3000 );
					break;
				default:
					//for revision < 2
					if( (pocl_state & STAT_CL_CLK_OFF_TIMER) < CRITICAL_CL_CLK_OFF_TIMER )
					{
						ADD_TIMER( 3000 );
						break;
					}
					ppo->Action = epcsConnectionSense;
					continue;
				}
				break;
			}
		case epcsClockDetect:
			{
#ifdef linux
				pocl_state = le32_to_cpu( ppo->Base->status );
#else
				pocl_state = ppo->Base->status;
#endif
				if( (pocl_state & STAT_CL_CLK_OFF_TIMER) >= CRITICAL_CL_CLK_OFF_TIMER )
				{
#ifdef DRIVER
					DbgPrint ( " %s, id %d, base %p status 0x%x camera absent\n", __FUNCTION__, ppo->register_index, ppo->Base, pocl_state );
#else
					printk( " %s, id %d, base %p status 0x%x camera absent\n", __FUNCTION__, ppo->register_index, ppo->Base, pocl_state );
#endif
					ppo->CLClockLost = TRUE;
					CANCEL_TIMER();
					ppo->Action = epcsConnectionSense;
					continue;
				}
#if !ENABLE_INTERRUPT
				ADD_TIMER( 3000 );
#endif
				break;
			}
		}
		break;
	}
}

#ifdef linux
//----------------------------------------------------------------------------------
void jit_timer_pocl( unsigned long arg )
//----------------------------------------------------------------------------------
{
	HandlePoCLEvent( (POCL_OBJECT*)arg );
}
#else
//----------------------------------------------------------------------------------
VOID DpcForPoCL( IN PKDPC Dpc, IN PVOID DeferredContext, IN PVOID SystemArgument1, IN PVOID SystemArgument2 )
//----------------------------------------------------------------------------------
{
	HandlePoCLEvent( (POCL_OBJECT*)DeferredContext );
}
#endif


//----------------------------------------------------------------------------------
unsigned char GetCLClockStatus( POCL_OBJECT *ppo )
//----------------------------------------------------------------------------------
{
	unsigned char clClockLost = ppo->CLClockLost;
	ppo->CLClockLost = FALSE;
	return clClockLost;
}
